@@ -129,6 +129,10 @@ class BDF(object):
     # match up with the sizes above due to how BDF defines the 'size'
     # attribute..
     bin_dtype = {
+            'flags':           numpy.int32,
+            'actualTimes':     numpy.int64,
+            'actualDurations': numpy.int64,
+            'zeroLags':        numpy.float32,
             'autoData':        numpy.float32,
             'crossData':       numpy.complex64,
             }
@@ -233,15 +237,21 @@ class BDF(object):
         # but seems convenient to do it here.
         cross_offset = 0
         auto_offset = 0
+        cross_meta_offset = 0
+        auto_meta_offset = 0
         self.spws = []
         for bb in self.sdmDataHeader.dataStruct.baseband:
             bbname = bb.attrib['name']
             self.basebands.append(bbname)
             for spw_elem in bb.spectralWindow:
                 # Build a list of spectral windows for each baseband
-                spw = BDFSpectralWindow(spw_elem, cross_offset, auto_offset)
+                spw = BDFSpectralWindow(spw_elem, cross_offset, auto_offset,
+                        cross_meta_offset=cross_meta_offset,
+                        auto_meta_offset=auto_meta_offset)
                 cross_offset += spw.dsize('cross')
                 auto_offset += spw.dsize('auto')
+                cross_meta_offset += spw.msize('cross')
+                auto_meta_offset += spw.msize('auto')
                 self.spws.append(spw)
 
     def get_integration(self, idx):
@@ -319,6 +329,21 @@ class BDF(object):
             result /= float(nsubout)
         return result
 
+    def get_meta(self, component, spwidx='all', corr='cross', trange=None):
+        if trange is None:
+            i0 = 0
+            i1 = self.numIntegration
+        else:
+            i0 = trange[0]
+            i1 = trange[1]
+        nsubout = i1 - i0
+        subdat = self.get_integration(i0).get_meta(component, spwidx, corr)
+        dshape = (nsubout,) + subdat.shape
+        result = numpy.zeros(dshape, dtype=subdat.dtype)
+        for i in range(i0, i1):
+            dat = self.get_integration(i).get_meta(component, spwidx, corr)
+            result[i-i0] = dat
+        return result
 
 class BDFSpectralWindow(object):
     """Class that represents spectral window information present in BDF files,
@@ -345,7 +370,7 @@ class BDFSpectralWindow(object):
 
     def __init__(self, spw_elem, cross_offset=None, auto_offset=None,
                  numBin=None, numSpectralPoint=None, sw=None, swbb=None,
-                 npol=None):
+                 npol=None, cross_meta_offset=None, auto_meta_offset=None):
         if spw_elem is not None:
             self._attrib = spw_elem.attrib
         else:
@@ -369,6 +394,9 @@ class BDFSpectralWindow(object):
 
         self.cross_offset = cross_offset
         self.auto_offset = auto_offset
+        self.cross_meta_offset = cross_meta_offset
+        self.auto_meta_offset = auto_meta_offset
+
 
     def to_xml(self):
         # Property?
@@ -433,6 +461,16 @@ class BDFSpectralWindow(object):
         data elements (real for auto, complex for cross)."""
         return numpy.product(self.dshape(type))
 
+    def mshape(self, corr):
+        """Return shape tuple of metadata array for this spectral window
+        and correlation type (cross or auto)."""
+        return (self.numBin, len(self.pols(corr)))
+
+    def msize(self, corr):
+        """Return size of metadata array for this spectral window and
+        correlation type (cross or auto)."""
+        return numpy.product(self.mshape(corr))
+
     @staticmethod
     def dims_match(spwlist, type):
         """Given a list of BDFSpectralWindow objects, return true if all
@@ -487,11 +525,13 @@ class BDFIntegration(object):
             bsize = bdf.bin_size[btype]  # size of the binary blob in bytes
             baxes = self.bin_axes[btype]
             # Determine outer step size for the array, either baselines
-            # antennas of baseline+antenna.  We can't apply the other
+            # antennas or baseline+antenna.  We can't apply the other
             # dimensions here because the number of elements can vary
             # per spw.
             if baxes[0] == 'BAL' and baxes[1] == 'ANT':
-                shape = (self.numBaseline+self.numAntenna, -1)
+                # In this case, cross and auto may have different step sizes
+                # so nothing we can do here.
+                shape = (-1,)
             elif baxes[0] == 'BAL':
                 shape = (self.numBaseline, -1)
             elif baxes[0] == 'ANT':
@@ -556,6 +596,48 @@ class BDFIntegration(object):
         dshape = (-1,) + spw.dshape(type)
         return self.data[loc][:, offs:offs+dsize].reshape(dshape)
 
+    def get_meta(self, component, spwidx='all', corr='cross'):
+        """
+        Return metadata from the requested binary component (flags, actualDurations, 
+        actualTimes).
+        """
+        # Assume the axes BAL ANT BAB SPW BIN STO
+        # Note for this, npol==3 actually means 3 unlike for the data
+        
+        # Normalize corr input
+        csize = self.numBaseline*sum([spw.msize('cross') for spw in self.spws])
+        if corr[0].lower() == 'c':
+            corr = 'cross'
+            n0 = self.numBaseline
+            offs = 0
+            tsize = csize
+        elif corr[0].lower() == 'a':
+            corr = 'auto'
+            n0 = self.numAntenna
+            offs = csize
+            tsize = self.numAntenna*sum([spw.msize('auto') for spw in self.spws])
+        else:
+            raise RuntimeError('Unsupported data type')
+
+        data = self.data[component][offs:offs+tsize].reshape((n0,-1))
+
+        if spwidx=='all':
+            if not BDFSpectralWindow.dims_match(self.spws, corr):
+                raise RuntimeError('BDFIntegration: ' +
+                                   'mixed array dimensions, spws must be ' +
+                                   'retrieved indivdually')
+            dshape = (n0, len(self.spws)) + self.spws[0].mshape(corr)
+            return data.reshape(dshape)
+
+        spw = self.spws[spwidx]
+        dshape = (n0,) + spw.mshape(corr)
+        dsize = spw.msize(corr)
+        if corr=='cross':
+            spwoffs = spw.cross_meta_offset
+        elif corr=='auto':
+            spwoffs = spw.auto_meta_offset
+        return data[:,spwoffs:spwoffs+dsize].reshape(dshape)
+
     def zerofraction(self, spwidx='all', type='cross'):
         """Returns the fraction of data points in the integration that
         are exactly zero (generally this means they have been flagged
